@@ -60,14 +60,14 @@
     AC_CHECK_TYPES([ptrdiff_t])      defines HAVE_PTRDIFF_T
     AC_CHECK_SIZEOF([unsigned long]) defines SIZEOF_UNSIGNED_LONG
     AC_CHECK_SIZEOF([void *])        defines SIZEOF_VOID_P
-                                             HAVE_LANGINFO_THOUSANDS_SEP
+                                             HAVE_LANGINFO_THOUSEP
 
     Note that the file estream.m4 provides the autoconf macro
     ESTREAM_PRINTF_INIT which runs all required checks.
     See estream-printf.h for ways to tune this code.
 
   Missing stuff:  wchar and wint_t
-                  thousands_sep in pr_float.
+                  thousep in pr_float.
 
 */
 
@@ -77,9 +77,6 @@
 
 #if defined(_WIN32) && !defined(HAVE_W32_SYSTEM)
 # define HAVE_W32_SYSTEM 1
-# if defined(__MINGW32CE__) && !defined (HAVE_W32CE_SYSTEM)
-#  define HAVE_W32CE_SYSTEM
-# endif
 #endif
 
 #include <stdio.h>
@@ -89,18 +86,14 @@
 #include <stdarg.h>
 #include <errno.h>
 #include <stddef.h>
-#include <assert.h>
 #if defined(HAVE_INTMAX_T) || defined(HAVE_UINTMAX_T)
 # ifdef HAVE_STDINT_H
 #  include <stdint.h>
 # endif
 #endif
-#ifdef HAVE_LANGINFO_THOUSANDS_SEP
+#ifdef HAVE_LANGINFO_THOUSEP
 #include <langinfo.h>
 #endif
-#ifdef HAVE_W32CE_SYSTEM
-#include <gpg-error.h>  /* ERRNO replacement.  */
-#endif
 #ifdef _ESTREAM_PRINTF_EXTRA_INCLUDE
 # include _ESTREAM_PRINTF_EXTRA_INCLUDE
 #endif
@@ -117,11 +110,7 @@
 #endif
 
 /* A wrapper to set ERRNO.  */
-#ifdef HAVE_W32CE_SYSTEM
-# define _set_errno(a)  gpg_err_set_errno ((a))
-#else
-# define _set_errno(a)  do { errno = (a); } while (0)
-#endif
+#define _set_errno(a)  do { errno = (a); } while (0)
 
 
 /* Calculate array dimension.  */
@@ -936,8 +925,8 @@ pr_integer (estream_printf_out_t outfnc, void *outfncarg,
     {
       int grouping = -1;
       const char * grouping_string =
-#ifdef HAVE_LANGINFO_THOUSANDS_SEP
-        nl_langinfo(THOUSANDS_SEP);
+#ifdef HAVE_LANGINFO_THOUSEP
+        nl_langinfo(THOUSEP);
 #else
         "'";
 #endif
@@ -1202,26 +1191,36 @@ pr_string (estream_printf_out_t outfnc, void *outfncarg,
   int rc;
   size_t n;
   const char *string, *s;
+  char *stringbuf = NULL;
 
   if (arg->vt != VALTYPE_STRING)
     return -1;
+
   string = value.a_string;
+
+  /* Make sure STRING is nul terminated, for call of string filter.  */
+  if (string && arg->precision >= 0)
+    {
+      /* STRING may be nul terminated shorter then the PRECISION.  */
+      for (n=0,s=string; n < arg->precision && *s; s++)
+        n++;
+
+      stringbuf = my_printf_realloc (NULL, n+1);
+      if (!stringbuf)
+        return -1;
+      memcpy (stringbuf, string, n);
+      stringbuf[n] = 0;
+      string = stringbuf;
+    }
+
   if (sf)
-    string = sf (value.a_string, string_no, sfvalue);
-  else
-    string = value.a_string;
+    string = sf (string, string_no, sfvalue);
 
   if (!string)
     string = "(null)";
-  if (arg->precision >= 0)
-    {
-      /* Test for nul after N so that we can pass a non-nul terminated
-         string.  */
-      for (n=0,s=string; n < arg->precision && *s; s++)
-        n++;
-    }
-  else
-    n = strlen (string);
+  n = strlen (string);
+  if (arg->precision >= 0 && n >= arg->precision)
+    n = arg->precision;
 
   if (!(arg->flags & FLAG_LEFT_JUST)
       && arg->width >= 0 && arg->width > n )
@@ -1248,7 +1247,9 @@ pr_string (estream_printf_out_t outfnc, void *outfncarg,
 
  leave:
   if (sf) /* Tell the filter to release resources.  */
-    sf (value.a_string, -1, sfvalue);
+    sf (string, -1, sfvalue);
+  if (stringbuf)
+    my_printf_realloc (stringbuf, 0);
 
   return rc;
 }
@@ -1405,13 +1406,13 @@ do_format (estream_printf_out_t outfnc, void *outfncarg,
       s += arg->length;
       format = s;
 
-      assert (argidx < argspecs_len);
+      gpgrt_assert (argidx < argspecs_len);
       argidx++;
 
       /* Apply indirect field width and precision values.  */
       if (arg->width == STAR_FIELD_VALUE)
         {
-          assert (valuetable[arg->width_pos-1].vt == VALTYPE_INT);
+          gpgrt_assert (valuetable[arg->width_pos-1].vt == VALTYPE_INT);
           arg->width = valuetable[arg->width_pos-1].value.a_int;
           if (arg->width < 0)
             {
@@ -1421,7 +1422,7 @@ do_format (estream_printf_out_t outfnc, void *outfncarg,
         }
       if (arg->precision == STAR_FIELD_VALUE)
         {
-          assert (valuetable[arg->precision_pos-1].vt == VALTYPE_INT);
+          gpgrt_assert (valuetable[arg->precision_pos-1].vt == VALTYPE_INT);
           arg->precision = valuetable[arg->precision_pos-1].value.a_int;
           if (arg->precision < 0)
             arg->precision = NO_FIELD_VALUE;
@@ -1431,13 +1432,13 @@ do_format (estream_printf_out_t outfnc, void *outfncarg,
         value.a_string = strerror (myerrno);
       else
         {
-          assert (arg->vt == valuetable[arg->arg_pos-1].vt);
+          gpgrt_assert (arg->vt == valuetable[arg->arg_pos-1].vt);
           value = valuetable[arg->arg_pos-1].value;
         }
 
       switch (arg->conspec)
         {
-        case CONSPEC_UNKNOWN: assert (!"bug"); break;
+        case CONSPEC_UNKNOWN: gpgrt_assert (!"bug"); break;
 
         case CONSPEC_DECIMAL:
         case CONSPEC_UNSIGNED:
@@ -1864,7 +1865,7 @@ _gpgrt_estream_vasprintf (char **bufp, const char *format, va_list arg_ptr)
       *bufp = NULL;
       return -1;
     }
-  assert (parm.used);   /* We have at least the terminating Nul.  */
+  gpgrt_assert (parm.used);   /* We have at least the terminating Nul.  */
   *bufp = parm.buffer;
   return parm.used - 1; /* Do not include that Nul. */
 }
